# Histogram Equalization
# Ver. 0.01 (2024_03)
# (c) 2024, Ohnishi, Yasuo
from ij import IJ
from ij import WindowManager as WM
from ij.io import OpenDialog
from loci.plugins import BF
from ij.plugin import ChannelSplitter
from ij.plugin import ContrastEnhancer
from ij.plugin import Duplicator
import os
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')

#----------------
def equalize(imp):
    imp2 = Duplicator().run(imp)
    #ContrastEnhancer().setNormalize(True)
    ContrastEnhancer().setUseStackHistogram(False)
    ContrastEnhancer().equalize(imp2)
    return imp2

#
window = WM.getActiveWindow()
if not window:
    od = OpenDialog("Choose a file")
    folder = od.getDirectory()
    filename = od.getFileName()
    if filename:
        path = folder + filename
        IJ.log(path)
        IJ.log(folder)
        IJ.log(filename)
# Always Open file with Bio-format Plugin
        imps = BF.openImagePlus(folder + filename) 
        imp0 = imps[0] #imp0: Original Image 
        imp0.show()
    else:
        IJ.showMessage("File is not selected.")
else: # target image is already open
    imp0 = IJ.getImage()
    imagename = imp0.getTitle()
    try:
        folder = imp0.getOriginalFileInfo().directory
    except:
        folder = IJ.getDirectory("Output_directory")

bitdepth = imp0.getBitDepth()
stackSize =imp0.getStackSize()
if bitdepth == 8:
    Mode = "8-bit"
elif bitdepth ==16:
    Mode = "16-bit"
elif bitdepth ==32:
    Mode = "32-bit"
else:
    Mode = "16-bit"

if Mode == "32-bit":
    IJ.showMessage("This program can not be applicable to 32-bit image!")
else:
    IJ.showMessage("To execute classic histogram equalization, press ALT key!")
    if stackSize > 1:
        chName = ['R', 'G', 'B'] # Channel name
        impsCh1 = []
        impsCh = ChannelSplitter.split(imp0) #Channel Splitting of imp0
        for i in range(stackSize):
            impsCh1.append(equalize(impsCh[i]))
            impsCh1[i].setTitle(chName[i])
            impsCh1[i].show()
        imp1 = IJ.run("Merge Channels...", "c1=R c2=G c3=B create keep")
    else:
            chname = "Mono channel"
            imp1 = equalize(imp0)
    IJ.saveAsTiff(imp1, folder + filename +"_equalized.tif")
    if stackSize > 1:
        for i in range(stackSize):
            impsCh[i].close() 
            impsCh1[i].hide() 
            impsCh1[i].close() 
IJ.showMessage("Process finished.")
